/**
 * Copyright (c) 1996-2024, SR Research Ltd., All Rights Reserved
 *
 * For use by SR Research licencees only. Redistribution and use in source
 * and binary forms, with or without modification, are NOT permitted.
 *
 * Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in
 * the documentation and/or other materials provided with the distribution.
 *
 * Neither name of SR Research Ltd nor the name of contributors may be used
 * to endorse or promote products derived from this software without
 * specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS
 * IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*******************************************************************************
 * EYELINK EXPT SUPPORT: SDL PLATFORM (Coppied from WIN32 PLATFORM)			   *
 * by Suganthan Subramaniam                                                    *
 * EYELINK II: 3 February 2003                                                 *
 * Provides folder support						                               *
 *                                                                             *
 *                                                                             *
 **************************************** WARNING ******************************
 *                                                                             *
 * UNDER NO CIRCUMSTANCES SHOULD PARTS OF THESE FILES BE COPIED OR COMBINED.   *
 * This will make your code impossible to upgrade to new releases in the future*
 * and SR Research will not give tech support for reorganized code.            *
 *                                                                             *
 * This file should not be modified. If you must modify it, copy the entire    *
 * file with a new name, and change the the new file.                          *
 *******************************************************************************/


#define  _CRT_SECURE_NO_WARNINGS // for msvc, otherwise we get a lot of warnings.
#include <stdio.h>    // for vsprintf() function
#include <stdarg.h>
#include <string.h>
#include <stdlib.h>
#include <time.h>
#include <errno.h>

#ifdef __APPLE__ //MACOSX
#include <SDL_ttf/SDL_ttf.h>
#include <CoreFoundation/CoreFoundation.h>
#include <NSSystemDirectories.h>
#endif
#include "folder_support.h"
#ifdef _WIN32
#include <direct.h>
#include <io.h>
#if (_MSC_VER > 1200)
#define strtok_r strtok_s
#else

#endif
#define makedir(a) _mkdir(a)
#define access(a,b) _access(a,b)
#define F_OK 00
#define PATH_DELIMITER ";"


#else
#include <libgen.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>

#define min(x,y) (x>y)? y:x
#define _stricmp strcasecmp
#define PATH_DELIMITER ":"
#define makedir(a) mkdir(a,S_IRWXU)
#endif

static char output_folder_path[4096]="";
static char exe_folder_path[4096]="";

/*
 * Returns the path to the output folder.
 * For macos and linux, default output folder is ~/Documents/<exe name>_<date string>
 * For Windows default output folder is <user profile>/My Documents/<exe name>_<date string>
 */
const char * get_output_folder(void)
{
	return output_folder_path;
}

int set_output_folder(const char *path)
{
	strcpy(output_folder_path,path);
	return 0;
}

static int recursive_mkdir(const char *dir) {
	char tmp[4096];
	char *p = NULL;
	size_t len;
	int rv;


	sprintf(tmp,"%s",dir);
	len = strlen(tmp);
	if(tmp[len - 1] == '/')
		tmp[len - 1] = 0;
	for(p = tmp + 1; *p; p++)
		if(*p == '/') {

			*p = 0;
			rv =makedir(tmp);
			if(rv !=0)
			{
				if(errno != EEXIST)
				{
					printf("Failed to create folder %s %d\n", tmp, rv);
					return rv;
				}
			}
			*p = '/';
		}
	rv= makedir(tmp);
	if (rv != 0)
	{
		if (errno != EEXIST)
		{
			printf("Failed to create folder %s %d\n", tmp, rv);
			return rv;
		}
	}
	return 0;
}

#ifdef _WIN32
#include <shlwapi.h>
#include <shlobj.h>

static const char * dirname(const char *path)
{
	static char result[2048];
	char drive[20];
	char dir[2048];
	_splitpath(path, drive, dir, NULL, NULL);
	if(drive[0])
		sprintf(result, "%s\\%s", drive, dir);
	else
		strcpy(result, dir);
	return result;
}
static const char * basename(const char *path)
{
	static char result[2048];
	_splitpath(path, NULL, NULL, result, NULL);
	return result;
}
static int getDocumentsFolder(char *doc)
{
	LPITEMIDLIST ppidl = NULL;
	HRESULT res = SHGetFolderLocation(NULL, CSIDL_MYDOCUMENTS, NULL, 0, &ppidl);
	if (res == S_OK)
	{
		SHGetPathFromIDList(ppidl, doc);
		ILFree(ppidl);
		return 1;
	}
	return 0;
}

static int getPublicDocumentsFolder(char *doc)
{
	LPITEMIDLIST ppidl = NULL;
	HRESULT res = SHGetFolderLocation(NULL, CSIDL_COMMON_DOCUMENTS, NULL, 0, &ppidl);
	if (res == S_OK)
	{
		SHGetPathFromIDList(ppidl, doc);
		ILFree(ppidl);
		return 1;
	}
	return 0;
}

static int try_creating_output_folder(const char *path, const char *exename, int make_sub)
{
	if(make_sub)
	{
		time_t current_time;
		struct tm * time_info;
		int len;
		char *outfolder;
		// output folder path prefix
		if (make_sub>1)
		{
			strcat(output_folder_path, "/Eyelink/SampleExperiments/");
			strcat(output_folder_path, exename);

		}
		strcat(output_folder_path, "/");
		//strcat(output_folder_path, exename); exe name alreay added
		// find the end fo the string to print time.
		len = strlen(output_folder_path);
		outfolder = output_folder_path + len;



		time(&current_time);
		time_info = localtime(&current_time);
		strftime(outfolder, sizeof(output_folder_path) - len, "%Y-%m-%d-%H-%M-%S", time_info);
	}
	return recursive_mkdir(output_folder_path);


}
#endif
/*
* This implementation is for the mac app. resource path is .app/Contents/Resources
*/
#if defined(__APPLE__) //defined(MACOSX) && defined(MACAPP)
const char * get_resources_path()
{
	static char wd[1024] = {0};
	if(!strlen(wd))
	{
		CFURLRef appRef = CFBundleCopyBundleURL(CFBundleGetMainBundle());
		CFStringRef appPath = CFURLCopyFileSystemPath(appRef,kCFURLPOSIXPathStyle);
		const char *pathPtr = CFStringGetCStringPtr(appPath,CFStringGetSystemEncoding());
		strcpy(wd,pathPtr);
		strcat(wd,"/Contents/Resources");
		return wd;
	}
	return wd;
}
#else
/* For windows and linux
*/

const char * get_resources_path()
{
	return exe_folder_path;
}
#endif

#ifdef _WIN32
static int OSCopyFile(const char* source, const char* destination)
{
	BOOL cancel = FALSE;
	return CopyFileEx(source, destination, NULL, NULL, &cancel, COPY_FILE_FAIL_IF_EXISTS);

}
#else
#if defined(__APPLE__) || defined(__FreeBSD__)
#include <copyfile.h>
#else
#include <sys/sendfile.h>
#endif

// copied from https://stackoverflow.com/questions/2180079/how-can-i-copy-a-file-on-unix-using-c
static int OSCopyFile(const char* source, const char* destination)
{
	int input, output;
	if ((input = open(source, O_RDONLY)) == -1)
	{
		printf("failed to open source file '%s' \n",  source);
		return -1;
	}
	if ((output = creat(destination, 0660)) == -1)
	{
		printf("failed to open destination file '%s'\n",  destination);
		close(input);
		return -1;
	}

	//Here we use kernel-space copying for performance reasons
#if defined(__APPLE__) || defined(__FreeBSD__)
	//fcopyfile works on FreeBSD and OS X 10.5+
	int result = fcopyfile(input, output, 0, COPYFILE_ALL);
#else
	//sendfile will work with non-socket output (i.e. regular file) on Linux 2.6.33+
	off_t bytesCopied = 0;
	struct stat fileinfo = {0};
	fstat(input, &fileinfo);
	int result = sendfile(output, input, &bytesCopied, fileinfo.st_size);
	if(result<0)
		printf("sendfile failed!. Error:%d Source:%s Destination:%s\n", result, source, destination);
#endif

	close(input);
	close(output);

	return result;
}
#endif

int copy_resource_to_output_folder(const char *resource)
{
	// for now stub. set the following link for more copy options.
	//https://stackoverflow.com/questions/2180079/how-can-i-copy-a-file-on-unix-using-c
	char src[2048];
	char dest[2048];
	const char *destfolder;
	sprintf(src,"%s/%s", get_resources_path(), resource);
	sprintf(dest,"%s/%s", get_output_folder(), resource);

	destfolder = dirname(dest);
	recursive_mkdir(destfolder);
	// recopy. the dirname reset the buffers.
	sprintf(src,"%s/%s", get_resources_path(), resource);
	sprintf(dest,"%s/%s", get_output_folder(), resource);
	return OSCopyFile(src,dest);
}

int open_output_folder(void)
{
#if defined(__APPLE__)
	char opencmd[2048];
	sprintf(opencmd,"open %s", get_output_folder());
	return system(opencmd);
#elif defined(_WIN32)
	return (int)ShellExecute(NULL, NULL, get_output_folder(), NULL, NULL, SW_SHOWNORMAL);
#else
	char opencmd[2048];
	printf("Output folder: %s \n",   get_output_folder());
	sprintf(opencmd,"xdg-open %s", get_output_folder());
	return system(opencmd);
#endif
}

#if defined(_WIN32)
int initialize_folder(const char *exe_path, const char *output_folder, int create_output_folder)
{
	char exename[256];
	strcpy(exename,basename((char *)exe_path));
	strcpy(exe_folder_path, dirname((char *)exe_path));
	if(output_folder)
		strcpy(output_folder_path, output_folder);
	if(create_output_folder)
	{
		if(output_folder_path[0] && !try_creating_output_folder(output_folder_path, exename, 0)) // check the give output folder
		   return 1;
		memset(output_folder_path,0,sizeof(output_folder_path));
		if(_getcwd(output_folder_path,sizeof(output_folder_path))) // check current folder
		   {
			   if(output_folder_path[0] && !try_creating_output_folder(output_folder_path, exename,1))
				   return 1;
		   }
		memset(output_folder_path,0,sizeof(output_folder_path));
		if(getPublicDocumentsFolder(output_folder_path) && !try_creating_output_folder(output_folder_path, exename,2)) //check the public documents folder
			return 1;
		memset(output_folder_path,0,sizeof(output_folder_path));
		if(getDocumentsFolder(output_folder_path) && !try_creating_output_folder(output_folder_path, exename,2)) // chec the Documents folder
			return 1;
		return 0;
	}
	return 1;
}
#else
// for mac and linux
/*
 *initializes the output folder.
 *returns non zero on success. 0 if failed.
 */
int initialize_folder(const char *_exe_path, const char *output_folder, int create_output_folder)
{
	char exe_path[4096]; //dirname and basename modifies the buffer
	char * homepath = getenv("HOME");
	strcpy(exe_path, _exe_path);
	strcpy(exe_folder_path, dirname((char *)exe_path));
	if(homepath)
	{

		if(output_folder)
		{
			strcpy(output_folder_path, output_folder);
			if(create_output_folder)
			{
				if(recursive_mkdir(output_folder_path))
					return 0;
			}
		}
		else
		{
			// output folder path prefix
			strcpy(output_folder_path, homepath);
			strcat(output_folder_path,"/Documents/Eyelink/SampleExperiments/");
			strcpy(exe_path, _exe_path);// reset the buffer
			strcat(output_folder_path,basename((char *)exe_path));
			strcat(output_folder_path,"/");

			// find the end fo the string to print time.
			int len =strlen(output_folder_path);
			char *outfolder =output_folder_path + len;
			time_t current_time;
			struct tm * time_info;


			time(&current_time);
			time_info = localtime(&current_time);
			strftime(outfolder, sizeof(output_folder_path)-len, "%Y-%m-%d-%H-%M-%S", time_info);
			if(create_output_folder)
			{
				if(recursive_mkdir(output_folder_path))
					return 0;
			}
		}

	}
	return 1;
}
#endif