/**
 * Copyright (c) 1996-2024, SR Research Ltd., All Rights Reserved
 *
 * For use by SR Research licencees only. Redistribution and use in source
 * and binary forms, with or without modification, are NOT permitted.
 *
 * Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in
 * the documentation and/or other materials provided with the distribution.
 *
 * Neither name of SR Research Ltd nor the name of contributors may be used
 * to endorse or promote products derived from this software without
 * specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS
 * IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*****************************************************************************
 * Windows sample experiment in C                                            *
 * For use with Version 2.0 of EyeLink Windows API                           *
 *****************************************************************************/

#include <stdlib.h>
#include "broadcast.h"
#include <sdl_text_support.h>
#include <gazecursor.h>



/* functions to map gaze from tracker to local pixels */
extern INT16 track2local_x(float x);
extern INT16 track2local_y(float y);


/********* PERFORM AN EXPERIMENTAL TRIAL  *******/

int record_mode_display(void)
{
  ALLF_DATA evt;
  UINT16 key;
  int eye_used = -1;   /* which eye to show gaze for */
  float x, y;	       /* gaze position  */
  float ox=-1, oy=-1;  /* old gaze position (to determine change)  */

  /* create font for position display */
  get_new_font(NULL, SCRWIDTH/50, 0);
  while(getkey()); /* dump any pending local keys  */

  /*enable link data reception without changing tracker mode */
  eyelink_reset_data(1);
  initialize_cursor(window, SCRWIDTH/50);
  eyelink_data_switch(RECORD_LINK_SAMPLES | RECORD_LINK_EVENTS);

  while(1)    /* loop while in record mode */
    {

      if(eyelink_tracker_mode() != EL_RECORD_MODE) break;
      key = getkey();            /* Local keys/abort test */
      if(key==TERMINATE_KEY)     /* test ALT-F4 or end of execution */
         break;
      else if(key)             /* OTHER: echo to tracker for control */
		eyelink_send_keybutton(key,0,KB_PRESS);

               /* CODE FOR PLOTTING GAZE CURSOR  */
      if(eyelink_newest_float_sample(NULL)>0)  /* new sample? */
		{
			eyelink_newest_float_sample(&evt);  /* get the sample data */
			if(eye_used == -1)   /* set which eye to track by first sample */
			{
				eye_used = eyelink_eye_available();
				if(eye_used == BINOCULAR)  /* use left eye if both tracked */
					eye_used = LEFT_EYE;
			}
			else
			{
				x = evt.fs.gx[eye_used];   /* get gaze position from sample  */
				y = evt.fs.gy[eye_used];
				if(x!=MISSING_DATA && y!=MISSING_DATA &&
					evt.fs.pa[eye_used]>0)  /* plot if not in blink */
                {                        /* plot in local coords   */
					draw_gaze_cursor(track2local_x(x),
                                    track2local_y(y));
                              /* report gaze position (tracker coords) */

                     {
					   SDL_Rect r = {(INT16)(SCRWIDTH*0.87), 0, (INT16)(window->w -SCRWIDTH*0.87), 50};
					   SDL_FillRect(window,&r,SDL_MapRGB(window->format,0, 0, 0));
                       graphic_printf(window, target_foreground_color, NONE, (int)(SCRWIDTH*0.87), 0, " %4.0f  ", x);
                       graphic_printf(window, target_foreground_color, NONE, (int)(SCRWIDTH*0.93), 0, " %4.0f  ", y);
                     }
                   ox = x;
                   oy = y;
                 }
				else
				{
					erase_gaze_cursor();   /* hide cursor during blink */
				}

				/* print tracker timestamp of sample */
				{
			   		SDL_Rect r = {(INT16)(SCRWIDTH*0.75), 0, (INT16)(SCRWIDTH*0.87 -SCRWIDTH*0.75), 50};
					SDL_FillRect(window,&r,SDL_MapRGB(window->format,0, 0, 0));
					graphic_printf(window,target_foreground_color, NONE, (int)(SCRWIDTH*0.75), 0, " % 8d ", evt.fs.time);
				}
			}
		}
    }
  erase_gaze_cursor();   /* erase gaze cursor if visible */
  return 0;
}

