/**
 * Copyright (c) 1996-2024, SR Research Ltd., All Rights Reserved
 *
 * For use by SR Research licencees only. Redistribution and use in source
 * and binary forms, with or without modification, are NOT permitted.
 *
 * Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in
 * the documentation and/or other materials provided with the distribution.
 *
 * Neither name of SR Research Ltd nor the name of contributors may be used
 * to endorse or promote products derived from this software without
 * specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS
 * IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*****************************************************************************
 * Windows sample experiment in C                                            *
 * For use with Version 2.0 of EyeLink Windows API                           *
 *****************************************************************************/


#include "eyedata.h"
#ifdef __APPLE__
#include <SDL_gfx/SDL_gfxPrimitives.h>
#else
#include <SDL/SDL_gfxPrimitives.h>
#endif
#include <sdl_text_support.h>

void drawMultipleSamples(int* iarray, int size)
{
	if(iarray)
	{
		int i=0;
		for(i=0; i<size-3;i+=2)
			lineRGBA(window,(Sint16)iarray[i], (Sint16)iarray[i+1],(Sint16)iarray[i+2], (Sint16)iarray[i+3],255,255,255,255);

		SDL_Flip(window);

		for(i=0; i<size-3;i+=2)
			lineRGBA(window,(Sint16)iarray[i], (Sint16)iarray[i+1],(Sint16)iarray[i+2], (Sint16)iarray[i+3],255,255,255,255);

	}
}





/* Plays back last trial's data; Prints white "F" for fixations; Connects samples with black line */
int playback_trial(void)
{
	//SDL_Color white = {0xF0, 0xF0, 0xF0};   /* define the drawing colors */
	SDL_Color grey  = {0xA0, 0xA0, 0xA0};
	SDL_Color black = {0, 0, 0};

	ALLF_DATA evt;	/* buffer for sample and event data */
	int eye_used = 0; /* which eye's data is used for plotting? */
	float x, y;		/* gaze position */
	//int last_sam_x = MISSING_DATA;  /* last sample position for drawing lines */
	//int last_sam_y = MISSING_DATA;
	int i;
	INT16 sample_rate = 250;
	unsigned int sample_count = 0;
    const unsigned int SAMPLEPLAYBACKOFFSET = 1;

	int* sarray = malloc(sizeof(int)*200);
	int prevFlag = 0;
	int countIdx =0, endIdx=0;

	memset(sarray, MISSING_DATA, 200*sizeof(int));

	/* Set up the display */
	clear_full_screen_window(grey);         /* erase display */


    get_new_font(NULL, 24,0);         /* select a font */

	graphic_printf(window, black, NONE, SCRWIDTH/2, 24, "Playing back last trial...");
	SDL_Flip(window);
	graphic_printf(window, black, NONE, SCRWIDTH/2, 24, "Playing back last trial..."); /* drawing to the background */

	set_offline_mode();           /* set up eye tracker for playback */
	eyelink_playback_start();     /* start data playback */

	/*   Wait for first data to arrive
	 Failure may mean no data or file not open

	 This function discards other data in file (buttons and messages)
	 until the start of recording.
	 If you need these events, then don't use this function.
	 Instead, wait for a sample or event before setting eye_data,
	 and have a timeout if no data is available in 2000 msec.
	 */
	if(!eyelink_wait_for_block_start(2000, 1, 1))
	{
		alert_printf("ERROR: playback data did not start!");
		return -1;
	}

	eye_used = eyelink_eye_available();
	if(eye_used==BINOCULAR) eye_used = LEFT_EYE;    /* use left eye if both available */

	/* determine sample rate */
	i = eyelink2_mode_data(&sample_rate, NULL, NULL, NULL);
	if(i==-1 || sample_rate<250) sample_rate = 250;  /* EyeLink I: sample rate = 4 msec */

	while(getkey()) {};     /* clear out any pending local keys */

	get_new_font(NULL, 20, 1);       /* set font for drawing fixation marks */
	while(1)   /* LOOP WHILE DATA AVAILABLE */
	{
		if(escape_pressed() || break_pressed()      /* exit if ESC, ALT-F4, or CTRL-C pressed */
		   || eyelink_last_button_press(NULL))  /* tracker button also exits */
		{
			clear_full_screen_window(target_background_color);  /* hide display */
			eyelink_playback_stop();                            /* stop playback */
			return 0;
		}
		/* PROCESS PLAYBACK DATA FROM LINK */
		i = eyelink_wait_for_next_data(NULL, 0, 0);   /* check for new data item */
		if(i==0)	                         /* 0: no new data */
		{                                /* Check if playback has completed */
			if((eyelink_current_mode() & IN_PLAYBACK_MODE)==0) break;
		}

		if(i == ENDFIX)	/* Was it a fixation event ? */
		{		/* PLOT FIXATIONS */
			eyelink_get_float_data(&evt);   /* get copy of fixation event */
			if(evt.fe.eye == eye_used)	  /* is it the eye we are plotting? */
			{                             /* Print a black "F" at average position */
				graphic_printf(window, black, NONE, (int)evt.fe.gavx, (int)evt.fe.gavy, "F");
				SDL_Flip(window);
				graphic_printf(window, black, NONE, (int)evt.fe.gavx, (int)evt.fe.gavy, "F");
			}
		}

#ifdef LOST_DATA_EVENT       /* AVAILABLE IN V2.1 OR LATER DLL ONLY */
		else if(i == LOST_DATA_EVENT)
		{
			alert_printf("Lost data in sequence");
		}
#endif
		/* Process samples: connect with lines */
		else if(i==SAMPLE_TYPE)
		{
			eyelink_get_float_data(&evt);  /* get copy of sample */
			if(eye_used != -1)             /* do we know which eye yet? */
			{
				sample_count ++;
                if (sample_count % SAMPLEPLAYBACKOFFSET == 0){
                    //msec_delay(SAMPLEPLAYBACKOFFSET*1000/sample_rate);  /* delay for real-time playback */
                    x = evt.fs.gx[eye_used];   /* get gaze position from sample */
                    y = evt.fs.gy[eye_used];
                    if(x!=MISSING_DATA && y!=MISSING_DATA &&
                       evt.fs.pa[eye_used]>0 )   /* check if pupil is present */
                    {
						if(prevFlag && endIdx >1)
						{
							sarray[countIdx++] =  sarray[endIdx-1];
							sarray[countIdx++] =  sarray[endIdx];

							sarray[endIdx-1] = MISSING_DATA;
							sarray[endIdx] = MISSING_DATA;
							endIdx = 0; prevFlag = 0;
						}
						if(countIdx < 200)
						{
							sarray[countIdx++] = x;
							sarray[countIdx++] = y;

						}else if (countIdx == 200)
						{
							drawMultipleSamples(sarray,200);
							memset(sarray, MISSING_DATA, 200);
							endIdx = 199;
							prevFlag = 1;
							countIdx = 0;
						}

                    }
                    else	/* no pupil present: must be in blink */
                    {
						endIdx = countIdx -1;
						drawMultipleSamples(sarray,countIdx);
						memset(sarray, MISSING_DATA, 200);
						prevFlag = 1;
						countIdx = 0;
                    }
                }
			}
			else
			{ /* if we don't know which eye yet, check which eye present */
				eye_used = eyelink_eye_available();
				if(eye_used==BINOCULAR) eye_used = LEFT_EYE;  /* use left eye if both available */
			}
		}
	}         /* END OF PLAYBACK LOOP */

		//check if we still have data to draw
	if(countIdx>0)
		drawMultipleSamples(sarray,countIdx);

	eyelink_playback_stop();                  /* Stop data playback */

	get_new_font(NULL, 24, 0);       /* Print completion message */
	graphic_printf(window, black, NONE, SCRWIDTH/2, 50, "Done: Press any key to continue.");
	SDL_Flip(window);
	graphic_printf(window, black, NONE, SCRWIDTH/2, 50, "Done: Press any key to continue.");

	while(getkey())  {};                      /* wait for new button or key press */
	while(!getkey() && !eyelink_last_button_press(NULL)) {};

	free(sarray);
	return 0;
}

